/****************************************************/
/* Examples Program For "CP-JR ARM7 LPC2368"        */
/* Target MCU  : Philips ARM7-LPC2368               */
/*       	   : X-TAL : 12.00 MHz                  */
/*             : PLL Setup = MSEL(12),NSEL(1)		*/
/* Keil Editor : uVision3 V3.53a                    */
/* Compiler    : Keil Realview MDK V3.20	        */
/* Create By   : Eakachai Makarn (WWW.ETT.CO.TH)    */
/* Last Update : 12/November/2007                   */
/* Function    : Example Read RTC + Display on UART0*/
/****************************************************/
// Read/Write Internal RTC of LPC2368
// Display Result on UART0(115200,N,8,1)

#include "LPC23xx.H" 	                                	// LPC2368 MPU Register
#include <stdio.h>											// For Used Function printf 

#define MASKSEC 0x3F  										// Second 00..59   00000000:00000000:00xxxxxx
#define MASKMIN 0x3F00										// Minute 00..59   00000000:00xxxxxx:00000000
#define MASKHR  0x1F0000									// Hour 00..23	   000xxxxx:00000000:00000000

// UART Buffer
char uart0_buf[]; 											// "sprint" UART[0] Buffer

/* pototype  section */
void init_serial0 (void); 									// Initil UART-0
void putchar0(char ch);  									// Put Char To UART-0
char getchar0(void);  										// Get Char From UART-0
void print_uart0(void);										// Print String to UART0 

int main(void)
{  
  unsigned char Hour,Minute,Second,Last_Second;				// RTC Buffer Data
  init_serial0();		   									// Initilial UART0 = 9600,N,8,1
  sprintf(uart0_buf,"\n\n\nCP-JR ARM7 LPC236..TEST RTC\n"); 
  print_uart0();

  // Initial Internal RTC Function
  // Initial RTC Function
  RTC_CCR  = 0;												// Reset All Bit Control
  RTC_CCR |= 0x00000010; 									// CLKSRC = 1 = Used EXT 32.768 KHz
  RTC_CCR |= 0x00000002; 									// Reset   Clock
  RTC_CCR &= 0xFFFFFFFD;									// Release Reset
  RTC_CCR |= 0x00000001;									// Start RTC Clock	

  //HOUR = 0x00;
  //MIN  = 0x00;
  //SEC  = 0x00;
  Last_Second = 0x00;
 
  // Start Test Read RTC and Display on UART0 //
  while(1)
  { 
    do 														// Repeat Get Second until Second Change
	{		
	  Hour   = (RTC_CTIME0 & MASKHR)>>16;					// Read Hour
	  Minute = (RTC_CTIME0 & MASKMIN)>>8;		 			// Read Minute
	  Second = RTC_CTIME0 & MASKSEC;		  				// Read Second
    }
    while(Last_Second == Second);							// Repeat if Second Not Change
	
    Last_Second = Second;									// Update Current Second
          
    //************************************//
    // Display Clock = Hour:Minute:Second //
    //************************************//
    sprintf(uart0_buf,"\rReal Time Clock = ");				// Print Message String 
	print_uart0();
  	sprintf(uart0_buf," %2d : %2d : %2d",Hour,Minute,Second);	 
	print_uart0();
  }
}

/********************************/
/* Initial UART0 = 115200,N,8,1 */
/********************************/
void init_serial0 (void)  
{
  PINSEL0 &= 0xFFFFFF0F;									// Reset P0.2,P0.3 Pin Config
  PINSEL0 |= 0x00000010;									// Select P0.2 = TxD(UART0)
  PINSEL0 |= 0x00000040;									// Select P0.3 = RxD(UART0)
  U0LCR = 0x83;                            					// 8 bits, no Parity, 1 Stop bit
  U0DLL = 3;                               					// Baud 115200BPS for 12MHz PCLK Clock         
  U0FDR = 0x67;                            					// Fractional Divider           
  U0LCR = 0x03;                            					// DLAB = 0
}

/******************************/
/* Write Character To UART[0] */
/******************************/
void putchar0 (char ch)  
{
  if (ch == '\n')  
  {
    while (!(U0LSR & 0x20));  								// Wait TXD Buffer Empty
    U0THR = 0x0D;                          					// Write CR
  }

  while (!(U0LSR & 0x20));									// Wait TXD Buffer Empty
  U0THR = ch;
}

/******************************/
/* Get character From UART[0] */
/******************************/
char getchar0()  
{
  while (!(U0LSR & 0x01));									// Wait RXD Receive Data Ready
  return (U0RBR);											// Get Receice Data & Return
}

/***************************/
/* Print String to UART[0] */
/***************************/
void print_uart0(void)
{
  char *p;													// Pointer Buffer
  p = uart0_buf;											// UART Buffer
 
  do 														// Get char & Print Until null
  {
    putchar0(*p); 											// Write char to UART
    p++;													// Next char
  }
  while(*p != '\0');										// End of ASCII (null)
 
  return;
}
