/****************************************************/
/* Examples Program For "CP-JR ARM7 LPC2368"        */
/* Target MCU  : Philips ARM7-LPC2368               */
/*       	   : X-TAL : 12.00 MHz                  */
/*             : PLL Setup = MSEL(12),NSEL(1)		*/
/* Keil Editor : uVision3 V3.53a                    */
/* Compiler    : Keil Realview MDK V3.20	        */
/* Create By   : Eakachai Makarn (WWW.ETT.CO.TH)    */
/* Last Update : 12/November/2007                   */
/* Function    : Example Use UART0,2,3              */
/****************************************************/

#include <LPC23xx.H> 	                                	// LPC2368 MPU Register
#include <stdio.h>											// For Used Function printf 

// Pin I/O LED Control Maskbit
#define  DIR_RS485   0x00080000   							// P1.19(0000 0000 0000 x000 0000 0000 0000 0000)
#define  RXD_RS485() IOCLR1 = DIR_RS485						// RS485 Direction = 0 (Receive)
#define  TXD_RS485() IOSET1 = DIR_RS485						// RS485 Direction = 1 (Transmit)

// UART Buffer
char uart0_buf[]; 											// "sprint" UART[0] Buffer
char uart2_buf[]; 											// "sprint" UART[2] Buffer
char uart3_buf[]; 											// "sprint" UART[3] Buffer

/* pototype  section */
void init_serial0 (void); 									// Initil UART-0
void init_serial2 (void); 									// Initil UART-2
void init_serial3 (void); 									// Initil UART-3

void putchar0(int ch);  									// Put Char To UART-0
void putchar2(int ch);  									// Put Char To UART-2
void putchar3(int ch);  									// Put Char To UART-3
int  getchar0(void);  										// Get Char From UART-0
int  getchar2(void);  										// Get Char From UART-2
int  getchar3(void); 										// Get Char From UART-3
void print_uart0(void);										// Print String to UART0 
void print_uart2(void);										// Print String to UART2 
void print_uart3(void);										// Print String to UART3
void delay(unsigned long int);								// Delay Time Function

int main(void)
{  
  int uart_data;											// Character Receive Buffer

  init_serial0();		   									// Initilial UART0 = 9600,N,8,1
  init_serial2();
  init_serial3();
    
  // UART[0] Print String //
  sprintf(uart0_buf,"ET-JR ARM7 LPC2368 : UART[0] Demo\n\r"); 	// Print Message String 
  print_uart0();

  // UART[2] Print String //
  sprintf(uart2_buf,"ET-JR ARM7 LPC2368 : UART[2] Demo\n\r");	// Print Message String 
  print_uart2();

  // UART[3] Print String //
  sprintf(uart3_buf,"ET-JR ARM7 LPC2368 : UART[3] Demo\n\r");	// Print Message String 
  print_uart3();
 
  // Loop Receive & Echo Test //
  while(1)													// Loop Continue
  {
	// RS232 : UART[0]
    if(U0LSR & 0x01)										// Check UART[0] Receive
    {
      uart_data = getchar0();								// Wait Receive Byte From UART
      if(uart_data==0x0D)									// If Enter
      {
        sprintf(uart0_buf,"ET-JR ARM7 LPC2368 : UART[0]\n\r"); 	
        print_uart0();
      }
      putchar0(uart_data);									// Echo Data to UART[0]
    }

	// RS232 : UART[2]
    if(U2LSR & 0x01)										// Check UART[2] Receive
    {
      uart_data = getchar2();								// Wait Receive Byte From UART
      if(uart_data==0x0D)									// If Enter
      {
        sprintf(uart2_buf,"ET-JR ARM7 LPC2368 : UART[2]\n\r");	
        print_uart2();
      }
	  putchar2(uart_data);									// Echo Data to UART[2]
	}

	// RS422/485 : UART[3]
	if(U3LSR & 0x01)										// Check UART[3 Receive
    {
      uart_data = getchar3();								// Wait Receive Byte From UART
      if(uart_data==0x0D)									// If Enter
      {
        sprintf(uart3_buf,"ET-JR ARM7 LPC2368 : UART[3]\n\r");	
        print_uart3();
      }
	  putchar3(uart_data);									// Echo Data to UART[3]
	}

  }	  													

}

/********************************/
/* Initial UART0 = 115200,N,8,1 */
/********************************/
void init_serial0 (void)  
{
  PINSEL0 &= 0xFFFFFF0F;									// Reset P0.2,P0.3 Pin Config
  PINSEL0 |= 0x00000010;									// Select P0.2 = TxD(UART0)
  PINSEL0 |= 0x00000040;									// Select P0.3 = RxD(UART0)

  U0LCR = 0x83;                            					// 8 bits, no Parity, 1 Stop bit
  U0DLL = 3;                               					// Baud 115200BPS for 12MHz PCLK Clock         
  U0FDR = 0x67;                            					// Fractional Divider           
  U0LCR = 0x03;                            					// DLAB = 0       
}

/********************************/
/* Initial UART2 = 115200,N,8,1 */
/********************************/
void init_serial2 (void)  
{
  // xxxx xxxx 0101 xxxx xxxx xxxx xxxx xxxx
  PINSEL0 &= 0xFF0FFFFF;									// Reset P0.10,P0.11 Pin Config
  PINSEL0 |= 0x00100000;									// Select P0.10 = TxD(UART2)
  PINSEL0 |= 0x00400000;									// Select P0.11 = RxD(UART2)

  // xxxx xxx1 xxxx xxxx xxxx xxxx xxxx xxxx
  PCONP   |= 0x01000000;									// UART2 Power-ON

  U2LCR = 0x83;                            					// 8 bits, no Parity, 1 Stop bit
  U2DLL = 3;                               					// Baud 115200BPS for 12MHz PCLK Clock         
  U2FDR = 0x67;                            					// Fractional Divider           
  U2LCR = 0x03;                            					// DLAB = 0
}

/********************************/
/* Initial UART3 = 115200,N,8,1 */
/********************************/
void init_serial3 (void)  
{
  // xxxx xxxx xxxx xxxx xxxx xxxx xxxx 1010
  PINSEL0 &= 0xFFFFFFF0;									// Reset P0.0,P0.1 Pin Config
  PINSEL0 |= 0x00000002;									// Select P0.0 = TxD(UART3)
  PINSEL0 |= 0x00000008;									// Select P0.1 = RxD(UART3)

  // Config P1.19 = Output Control Direction RS485
  // P1.19 = "0" = Receive  RS485
  // P1.19 = "1" = Transmit	RS485
  // xxxx xxxx xxxx xxxx 00xx xxxx xxxx xxxx
  PINSEL3 &= 0xFFFF3FFF;									// P1.19 = GPIO
  IODIR1 = DIR_RS485;										// Pin Control Direction RS485 = Output
  RXD_RS485();												// Default RS485 Direction

  // xxxx xx1x xxxx xxxx xxxx xxxx xxxx xxxx
  PCONP   |= 0x02000000;									// UART3 Power-ON

  U3LCR = 0x83;                            					// 8 bits, no Parity, 1 Stop bit
  U3DLL = 3;                               					// Baud 115200BPS for 12MHz PCLK Clock         
  U3FDR = 0x67;                            					// Fractional Divider           
  U3LCR = 0x03;                           	 				// DLAB = 0
}

/******************************/
/* Write Character To UART[0] */
/******************************/
void putchar0 (int ch)  
{
  if (ch == '\n')  
  {
    while (!(U0LSR & 0x20));  								// Wait TXD Buffer Empty
    U0THR = 0x0D;                          					// Write CR
  }

  while (!(U0LSR & 0x20));									// Wait TXD Buffer Empty
  U0THR = ch;
}

/******************************/
/* Write Character To UART[2] */
/******************************/
void putchar2 (int ch)  
{
  if (ch == '\n')  
  {
    while (!(U2LSR & 0x20));  								// Wait TXD Buffer Empty
    U2THR = 0x0D;                          					// Write CR
  }

  while (!(U2LSR & 0x20));									// Wait TXD Buffer Empty
  U2THR = ch;
}

/******************************/
/* Write Character To UART[3] */
/******************************/
void putchar3 (int ch)  
{  
  if (ch == '\n')  
  {
    while (!(U3LSR & 0x20));  								// Wait TXD Buffer Empty
    U3THR = 0x0D;                          					// Write CR
  }

  while (!(U3LSR & 0x20));									// Wait TXD Buffer Empty
  U3THR = ch;
}

/******************************/
/* Get character From UART[0] */
/******************************/
int getchar0()  
{
  while (!(U0LSR & 0x01));									// Wait RXD Receive Data Ready
  return (U0RBR);											// Get Receice Data & Return
}

/******************************/
/* Get character From UART[2] */
/******************************/
int getchar2()  
{
  while (!(U2LSR & 0x01));									// Wait RXD Receive Data Ready
  return (U2RBR);											// Get Receice Data & Return
}

/******************************/
/* Get character From UART[3] */
/******************************/
int getchar3()  
{
  while (!(U3LSR & 0x01));									// Wait RXD Receive Data Ready
  return (U3RBR);											// Get Receice Data & Return
}

/***************************/
/* Print String to UART[0] */
/***************************/
void print_uart0(void)
{
  char *p;													// Pointer Buffer
  p = uart0_buf;											// UART Buffer
 
  do 														// Get char & Print Until null
  {
    putchar0(*p); 											// Write char to UART
    p++;													// Next char
  }
  while(*p != '\0');										// End of ASCII (null)
 
  return;
}

/***************************/
/* Print String to UART[2] */
/***************************/
void print_uart2(void)
{
  char *p;													// Pointer Buffer
  p = uart2_buf;											// UART Buffer
 
  do 														// Get char & Print Until null
  {
    putchar2(*p); 											// Write char to UART
    p++;													// Next char
  }
  while(*p != '\0');										// End of ASCII (null)
 
  return;
}

/***************************/
/* Print String to UART[3] */
/***************************/
void print_uart3(void)
{
  char *p;													// Pointer Buffer
  p = uart3_buf;											// UART Buffer

  TXD_RS485();												// Swap RS485 Direction = Transmit
  delay(50000);

  do 														// Get char & Print Until null
  {
    putchar3(*p); 											// Write char to UART
    p++;													// Next char
  }
  while(*p != '\0');										// End of ASCII (null)

  delay(50000);
  RXD_RS485();												// Swap RS485 Direction = Receive

  return;
}

/***********************/
/* Delay Time Function */
/*    1-4294967296     */
/***********************/
void delay(unsigned long int count1)
{
  while(count1 > 0) {count1--;}								// Loop Decrease Counter	
}

