/*----------------------------------------------------------------------------
 *      Name:    DEMO.C
 *      Purpose: USB HID Demo
 *      Version: V1.10
 *----------------------------------------------------------------------------
 *      This file is part of the uVision/ARM development tools.
 *      This software may only be used under the terms of a valid, current,
 *      end user licence from KEIL for a compatible version of KEIL software
 *      development tools. Nothing else gives you the right to use it.
 *
 *      Copyright (c) 2005-2006 Keil Software.
 *---------------------------------------------------------------------------*/

#include <LPC23xx.H>                        /* LPC23xx definitions */

#include "type.h"
#include "usb.h"
#include "usbcfg.h"
#include "usbhw.h"
#include "usbcore.h"
#include "demo.h"


BYTE InReport;                              /* HID Input Report    */
                                            /*   Bit0   : Buttons  */
                                            /*   Bit1..7: Reserved */

BYTE OutReport;                             /* HID Out Report      */
                                            /*   Bit0..7: LEDs     */


/*
 *  Get HID Input Report -> InReport
 */
void GetInReport (void) 
{
  InReport = 0x00;	/* Default All Input */

  /* Check if SW1(ISP LOAD) is pressed */
  if ((FIO2PIN & SW1) == 0) 
  {             
    InReport |= 0x01;
  } 
  else 
  {
    InReport &= ~0x01;
  }

  /* Check if SW3(P4[28]) is pressed */
  if ((FIO4PIN & SW3) == 0) 
  {             
    InReport |= 0x02;
  } 
  else 
  {
    InReport &= ~0x02;
  }

  /* Check if SW4(P4[29]) is pressed */
  if ((FIO4PIN & SW4) == 0) 
  {             
    InReport |= 0x04;
  } 
  else 
  {
    InReport &= ~0x04;
  }
}


/*
 *  Set HID Output Report <- OutReport
 */
void SetOutReport (void) 
{
  FIO2SET = LEDMSK;	  						/* Toggles Output */
  FIO2CLR = OutReport;						/* Active LED "0" */
}


/* Main Program */
int main (void) 
{
  PINSEL10 = 0;                             /* Disable ETM Interfaces */
  FIO2DIR = LEDMSK;                         /* P2[0..7] = Output only */
  USB_Init();                               /* USB Initialization */
  USB_Connect(TRUE);                        /* USB Connect */

  while (1);                                /* Loop forever */
}
